#!/usr/bin/perl
# distribution boxbackup-0.11_trunk_2979 (svn version: 2979)
# Box Backup, http://www.boxbackup.org/
# 
# Copyright (c) 2003-2010, Ben Summers and contributors.
# All rights reserved.
# 
# Note that this project uses mixed licensing. Any file with this license
# attached, or where the code LICENSE-GPL appears on the first line, falls
# under the "Box Backup GPL" license. See the file COPYING.txt for more
# information about this license.
# 
# ---------------------------------------------------------------------
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
# 
# [http://www.gnu.org/licenses/old-licenses/gpl-2.0.html#SEC4]
# 
# As a special exception to the GPLv2, the Box Backup Project gives
# permission to link any code falling under this license (the Box Backup
# GPL) with any software that can be downloaded from
# the OpenSSL website [http://www.openssl.org] under either the
# "OpenSSL License" or the "Original SSLeay License", and to distribute
# the linked executables under the terms of the "Box Backup GPL" license.
# 
# As a special exception to the GPLv2, the Box Backup Project gives
# permission to link any code falling under this license (the Box Backup
# GPL) with any version of Microsoft's Volume Shadow Copy Service 7.2 SDK
# or Microsoft Windows Software Development Kit (SDK), including
# vssapi.lib, that can be downloaded from the Microsoft website
# [*.microsoft.com], and to distribute the linked executables under the
# terms of the "Box Backup GPL" license.
use strict;

print "Creating built-in documentation for bbackupquery...\n";

open DOC,"documentation.txt" or die "Can't open documentation.txt file";
my $section;
my %help;
my @in_order;

while(<DOC>)
{
	if(m/\A>\s+(\w+)/)
	{
		$section = $1;
		m/\A>\s+(.+)\Z/;
		$help{$section} = $1."\n";
		push @in_order,$section;
	}
	elsif(m/\A</)
	{
		$section = '';
	}
	elsif($section ne '')
	{
		$help{$section} .= $_;
	}
}

close DOC;

open OUT,">autogen_Documentation.cpp" or die "Can't open output file for writing";

print OUT <<__E;
//
// Automatically generated file, do not edit.
//

#include "Box.h"

#include "MemLeakFindOn.h"

const char *help_commands[] =
{
__E

for(@in_order)
{
	print OUT qq:\t"$_",\n:;
}

print OUT <<__E;
	0
};

const char *help_text[] =
{
__E

for(@in_order)
{
	my $t = $help{$_};
	$t =~ s/\t/    /g;
	$t =~ s/\n/\\n/g;
	$t =~ s/"/\\"/g;
	print OUT qq:\t"$t",\n:;	
}

print OUT <<__E;
	0
};

__E

close OUT;
