// distribution boxbackup-0.11_trunk_2979 (svn version: 2979)
// Box Backup, http://www.boxbackup.org/
// 
// Copyright (c) 2003-2010, Ben Summers and contributors.
// All rights reserved.
// 
// Note that this project uses mixed licensing. Any file with this license
// attached, or where the code LICENSE-DUAL appears on the first line, falls
// under this license. See the file COPYING.txt for more information.
// 
// This file is dual licensed. You may use and distribute it providing that you
// comply EITHER with the terms of the BSD license, OR the GPL license. It is
// not necessary to comply with both licenses, only one.
// 
// The BSD license option follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//  
// 3. Neither the name of the Box Backup nor the names of its contributors may
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// [http://en.wikipedia.org/wiki/BSD_licenses#3-clause_license_.28.22New_BSD_License.22.29]
// 
// The GPL license option follows:
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
// 
// [http://www.gnu.org/licenses/old-licenses/gpl-2.0.html#SEC4]
// --------------------------------------------------------------------------
//
// File
//		Name:    RaidFileController.h
//		Purpose: Controls config and daemon comms for RaidFile classes
//		Created: 2003/07/08
//
// --------------------------------------------------------------------------

/*  NOTE: will log to local5: include a line like
	local5.info                                             /var/log/raidfile
	in /etc/syslog.conf
*/

#ifndef RAIDFILECONTROLLER__H
#define RAIDFILECONTROLLER__H

#include <string>
#include <vector>

// --------------------------------------------------------------------------
//
// Class
//		Name:    RaidFileDiscSet
//		Purpose: Describes a set of paritions for RAID like files.
//				 Use as list of directories containing the files.
//		Created: 2003/07/08
//
// --------------------------------------------------------------------------
class RaidFileDiscSet : public std::vector<std::string>
{
public:
	RaidFileDiscSet(int SetID, unsigned int BlockSize)
		: mSetID(SetID),
		  mBlockSize(BlockSize)
	{
	}
	RaidFileDiscSet(const RaidFileDiscSet &rToCopy)
		: std::vector<std::string>(rToCopy),
		  mSetID(rToCopy.mSetID),
		  mBlockSize(rToCopy.mBlockSize)
	{
	}
	
	~RaidFileDiscSet()
	{
	}

	int GetSetID() const {return mSetID;}
	
	int GetSetNumForWriteFiles(const std::string &rFilename) const;
	
	unsigned int GetBlockSize() const {return mBlockSize;}

	// Is this disc set a non-RAID disc set? (ie files never get transformed to raid storage)
	bool IsNonRaidSet() const {return 1 == size();}

private:
	int mSetID;
	unsigned int mBlockSize;
};

class _RaidFileController;	// compiler warning avoidance

// --------------------------------------------------------------------------
//
// Class
//		Name:    RaidFileController
//		Purpose: Manages the configuration of the RaidFile system, handles
//				 communication with the daemon.
//		Created: 2003/07/08
//
// --------------------------------------------------------------------------
class RaidFileController
{
	friend class _RaidFileController;	// to avoid compiler warning
private:
	RaidFileController();
	RaidFileController(const RaidFileController &rController);
public:
	~RaidFileController();
	
public:
	void Initialise(const std::string& rConfigFilename = 
		"/etc/boxbackup/raidfile.conf");
	int GetNumDiscSets() {return mSetList.size();}

	// --------------------------------------------------------------------------
	//
	// Function
	//		Name:    RaidFileController::GetController()
	//		Purpose: Gets the one and only controller object.
	//		Created: 2003/07/08
	//
	// --------------------------------------------------------------------------	
	static RaidFileController &GetController() {return mController;}
	RaidFileDiscSet &GetDiscSet(unsigned int DiscSetNum);

	static std::string DiscSetPathToFileSystemPath(unsigned int DiscSetNum, const std::string &rFilename, int DiscOffset);
	
private:
	std::vector<RaidFileDiscSet> mSetList;
	
	static RaidFileController mController;
};

#endif // RAIDFILECONTROLLER__H

