#!/usr/bin/perl
# distribution boxbackup-0.11_trunk_2979 (svn version: 2979)
# Box Backup, http://www.boxbackup.org/
# 
# Copyright (c) 2003-2010, Ben Summers and contributors.
# All rights reserved.
# 
# Note that this project uses mixed licensing. Any file with this license
# attached, or where the code LICENSE-GPL appears on the first line, falls
# under the "Box Backup GPL" license. See the file COPYING.txt for more
# information about this license.
# 
# ---------------------------------------------------------------------
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
# 
# [http://www.gnu.org/licenses/old-licenses/gpl-2.0.html#SEC4]
# 
# As a special exception to the GPLv2, the Box Backup Project gives
# permission to link any code falling under this license (the Box Backup
# GPL) with any software that can be downloaded from
# the OpenSSL website [http://www.openssl.org] under either the
# "OpenSSL License" or the "Original SSLeay License", and to distribute
# the linked executables under the terms of the "Box Backup GPL" license.
# 
# As a special exception to the GPLv2, the Box Backup Project gives
# permission to link any code falling under this license (the Box Backup
# GPL) with any version of Microsoft's Volume Shadow Copy Service 7.2 SDK
# or Microsoft Windows Software Development Kit (SDK), including
# vssapi.lib, that can be downloaded from the Microsoft website
# [*.microsoft.com], and to distribute the linked executables under the
# terms of the "Box Backup GPL" license.
use strict;

my $flags = $ARGV[0] or "";

unless(open IN,"../../bin/bbackupquery/bbackupquery -Wwarning " .
	"-c testfiles/bbackupd.conf " .
	"-l testfiles/query4.log " .
	"\"compare -ac$flags\" quit 2>&1 |")
{
	print "FAIL: opening compare utility\n";
	exit 2;
}

my $ret = 1;
my $seen = 0;

while(<IN>)
{
	next unless m/\S/;
	print "READ: $_";

	if (m/continousupdate/)
	{
		unless (/exists/)
		{
			print "FAIL: continousupdate line does not match\n";
			$ret = 2;
		}
		$seen = 1;
	}
	elsif (m/^No entry for terminal type/ or
		m/^using dumb terminal settings/)
	{
		# skip these lines, may happen in Debian buildd
		# with no terminal.
	}
	else
	{
		unless (/\AWARNING/ or /\ADifferences/ or /might be reason/
			or /probably due to file mod/)
		{
			print "FAIL: Summary line does not match\n";
			$ret = 2;
		}
	}
}

close IN;

unless ($seen)
{
	print "FAIL: missing line matching continousupdate\n";
	$ret = 2;
}

exit $ret;

